package com.sw.service.impl;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.sw.activemq.producer.IMessageProducerService;
import com.sw.domain.TMask;
import com.sw.service.SnatchMaskService;
import com.sw.service.TMaskService;
import com.sw.service.TOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description TODO
 * @Author DingLan
 * @Date 2022/2/18 13:22
 **/
@Service
public class SnatchMaskServiceImpl implements SnatchMaskService {
    @Autowired
    private TMaskService maskService;

    @Autowired
    private TOrderService orderService;

    @Resource
    private RedisTemplate redisTemplate;

    @Resource
    private IMessageProducerService mqService;

    @Value("${amount}")
    private Integer amount;

    @Override
    @Transactional
    public String doSnatch(Integer uid) {
        Integer maskId = 1;
        //1.get the amount of remaining masks
        TMask mask = maskService.getById(maskId);
        if(mask.getMaskStock() > amount){
            //3.create order
            int res = orderService.creatOrder(uid, maskId);
            //4.decrease the stock of mask
            Boolean flag = maskService.decrease(maskId, mask.getMaskStock() - amount);
            if(res > 0 && flag){

                //3. add to successUserUids
                redisTemplate.opsForSet().add("successUserUids", uid);
                return "恭喜你 抢购成功";
            }else{
                return  "抢购失败 稍后重试";
            }
        }
        return "库存不足 抢购失败";
    }

    /**
     * 查看redis库存是否充足
     */
    @Override
    public  Boolean checkAndDecrRemaining(Integer mid) {
        Object o = new Object();
        Integer remaining = 0;
//        synchronized (o) {
            remaining = Integer.valueOf(redisTemplate.opsForHash().get("usableMasks", "mask-" + mid).toString());
            if(remaining.compareTo(amount) >= 0) {
                Long res = redisTemplate.opsForHash().increment("usableMasks", "mask-" + mid, -amount);
                System.out.println("increment res : "+res +"---" + Thread.currentThread());//扣减之后的库存数量
                return true;
            }else{
                return false;
            }
//        }
    }

    @Override
    public String doSnatchMQ(Integer uid, Integer mid) {
        Map<String, Object> agrs = new HashMap<>();
        agrs.put("uid", uid);
        agrs.put("mid", mid);
        mqService.sendMessage(agrs);
        return "抢购成功 请稍后查看订单";
    }

    /**
     * mq消费端 调用方法 扣减sql库存 创建未支付订单
     * @param uid
     * @param maskId
     */
    @Override
    @Transactional
    public void doDecrAndCreateOrder(Integer uid, Integer maskId) {
        //1.get the amount of remaining masks
        TMask mask = maskService.getById(maskId);
        if(mask.getMaskStock() >= amount) {
            //3.create order
            int res = orderService.creatOrder(uid, maskId);
            //4.decrease the stock of mask
            Boolean flag = maskService.decrease(maskId, mask.getMaskStock() - amount);
            if (res > 0 && flag) {

                //3. add to successUserUids
                redisTemplate.opsForSet().add("successUserUids", uid);
            }
        }else {
            // TODO 如果sql中库存不够 扣减失败之后 消息已经消费了 或者是事物回滚了  如何处理？？？
            return;
        }
    }
}
